`timescale 1ns / 1ps
//*****************************************************************************
// File Name            : mb_mcs_sys.v
//-----------------------------------------------------------------------------
// Function             : MicroBlaze MAC system 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2013/04/19
// -.-- 2013/07/08//*****************************************************************************

module mb_mcs_sys(  clk_100m,Reset, UART_Rx, UART_Tx,led,scl,sda,dip_sw,lcd_co,
MTxClk,MTxD,MTxEn,MTxErr,MRxClk,MRxD,MRxDV,MRxErr,MColl,MCrs,Mdc,Mdio,phy_rstb,
,mon
    );

  input  clk_100m;      //input clock 100MHz
  input  Reset;    //reset Hi=reset
  input  UART_Rx;  //UART rx 115200bps
  output UART_Tx; //UART tx 115200bps
  output [3 : 0] led;   //led Hi=on
  input  [3 : 0] dip_sw; //input dip_sw
  inout          scl; //i2c clk
  inout          sda; //i2c data
  output         lcd_co;
  //ether
  input           MTxClk; 
  output   [3:0]  MTxD;   
  output          MTxEn;  
  output          MTxErr; 
  input           MRxClk; 
  input    [3:0]  MRxD;   
  input           MRxDV;  
  input           MRxErr; 
  input           MColl;  
  input           MCrs;   
  output          Mdc;   
  inout           Mdio; 
  output           phy_rstb; 
  output          mon;
   
  // 
  parameter p_reg_addr_low =  32'hC0000000;
  parameter p_reg_addr_hi  =  32'hC0000007;
  parameter p_bram_addr_low = 32'hC0002000;
  parameter p_bram_addr_hi  = 32'hC0003FFF;
  // 
  parameter p_wb_addr_low  =  32'hD0000000;
  parameter p_wb_addr_hi   =  32'hEFFFFFFF;
  parameter p_wb_pwm0_low   =  p_wb_addr_low + 0;
  parameter p_wb_pwm0_hi    =  p_wb_addr_low + 15;
  parameter p_wb_pwm1_low   =  p_wb_addr_low + 16;
  parameter p_wb_pwm1_hi    =  p_wb_addr_low + 31;
  parameter p_wb_i2c_low   =  p_wb_addr_low + 64;
  parameter p_wb_i2c_hi   =  p_wb_addr_low + 127;
  //parameter p_wb_bram_low =  p_wb_addr_low + 32'h0001000;
  //parameter p_wb_bram_hi =  p_wb_addr_low + 32'h00017ff;
  parameter p_wb_emac_low =  p_wb_addr_low + 32'h0001000;
  parameter p_wb_emac_hi =  p_wb_addr_low + 32'h00017ff;
   
  wire           MTxClk_i ;
  wire           MRxClk_i; 
  wire    [3:0]  MRxD_i;   
  wire           MRxDV_i;
   
  wire IO_Addr_Strobe;
  wire IO_Read_Strobe;
  wire IO_Write_Strobe;
  wire [31 : 0] IO_Read_Data;
  wire [31 : 0] IO_Read_Data_reg;
  wire [31 : 0] IO_Read_Data_bram;
  wire [31 : 0] IO_Read_Data_wb;
  wire [31 : 0] IO_Address;
  wire [3 : 0]  IO_Byte_Enable;
  wire [31 : 0] IO_Write_Data;
  wire IO_Ready;
  wire IO_Ready_reg;
  wire IO_Ready_bram;
  wire IO_Ready_wb;
  wire iobus_reg_sel;
  wire iobus_bram_sel;  
  wire iobus_wb_sel;   
  //timmer
  wire FIT1_Interrupt;
  wire FIT1_Toggle;    
  wire PIT1_Interrupt; 
  wire PIT1_Toggle; 
  //GPO   
  wire [31 : 0] GPO1;
  wire [31 : 0] GPI1;
  //GPI
  wire GPI1_Interrupt;
  //Interrupt controller
  wire INTC_Interrupt;
  wire INTC_IRQ;
  //iobus_reg
  wire [31:0] data_in;
  wire [3:0]  data_in_en;
  wire [31:0] data_out;
  wire [3:0]  data_out_en;
  //user_module
  wire [7:0] chk_data; 
  //block ram port-B
  wire      clkb; 
  wire  [3:0]web;
  wire  [10 : 0] addrb;
  wire  [31 : 0] dinb;
  wire  [31 : 0] doutb;
  //Wishbone bus microblaze -> IP
  wire         WB_CLK;         // clock 
  wire         WB_RST;         // reset (asynchronous active hi)
  wire         WB_CYC;         // cycle
  wire         WB_STB;         // strobe
  wire  [31:0] WB_ADR;         // address adr_i
  wire         WB_WE;          // write enable
  wire  [31:0] WB_DAT_I;          // data output
  wire  [31:0] WB_DAT_O;          // data input
  wire  [3:0]  WB_SEL;         // byte enable
  wire         WB_ACK;         // normal bus termination
  wire         WB_ERR;         // err bus termination
  wire         WB_RTY;         // rtry
  //pwm
  wire         cyc_i_pwm0;         // cycle
  wire         stb_i_pwm0;         // strobe
  wire         we_i_pwm0;          // write enable
  wire  [15:0] adr_i_pwm0;         // address adr_i
  wire  [15:0] dat_o_pwm0;   // data input
  wire         ack_o_pwm0;         // normal bus termination
  wire         wb_pwm0_sel;
  wire         cyc_i_pwm1;         // cycle
  wire         stb_i_pwm1;         // strobe
  wire  [15:0] adr_i_pwm1;         // address adr_i
  wire         we_i_pwm1;          // write enable
  wire  [15:0] dat_o_pwm1;   // data input
  wire         ack_o_pwm1;         // normal bus termination
  wire         wb_pwm1_sel;
  //i2c
  wire         cyc_i_i2c;         // cycle
  wire         stb_i_i2c;         // strobe
  wire         we_i_i2c;          // write enable
  wire  [2:0] adr_i_i2c;         // address adr_i
  wire  [7:0] dat_o_i2c;   // data input
  wire         ack_o_i2c;         // normal bus termination
  wire         wb_i2c_sel;
  wire         inta_i2c;
  /*             
  //bram
  wire         cyc_i_bram;         // cycle
  wire         stb_i_bram;         // strobe
  wire         we_i_bram;          // write enable
  wire  [31:0] dat_o_bram;   // data input
  wire         ack_o_bram;         // normal bus termination
  wire         err_o_bram;         // normal bus termination
  wire         wb_bram_sel;
  wire         inta_bram;
  */              
  //emac
  wire         cyc_i_emac;         // cycle
  wire         stb_i_emac;         // strobe
  wire         we_i_emac;          // write enable
  wire  [31:0] dat_o_emac;   // data input
  wire         ack_o_emac;         // normal bus termination
  wire         err_o_emac = 0;         // normal bus termination
  wire         wb_emac_sel;
  wire         inta_emac;
  //mdio   
  wire md_pad_i;      // MII data input (from I/O cell)
  wire mdc_pad_o;     // MII Management data clock (to PHY)
  wire md_pad_o;      // MII data output (to I/O cell)
  wire md_padoe_o;    // MII data output enable (to I/O cell)
   
  //opencores PWM signals
  wire  i_extclk;
  wire  [15:0]i_DC;
  wire  i_valid_DC;  
  wire  o_pwm0;   
   
  //opencores I2C signals
  wire rstn;
  wire scl_pad_i;
  wire scl_pad_o;
  wire scl_padoe_o;
  wire sda_pad_i;
  wire sda_pad_o;
  wire sda_padoe_o;
 
  //mon
  reg mon;
  //clk
  wire Clk;
  wire clk_25m;
  wire LOCKED; 
  wire reset_unlocked ;
  reg scl_mon ;
  reg scl_o_mon ;
  reg scl_oe_mon ;
  reg sda_mon ;
  reg sda_oe_mon ;
  reg sda_o_mon ;
  reg sda_i_mon ;
  reg scl_fd1; 
  reg scl_o_fd1 ;
  reg scl_oe_fd1 ;
  reg sda_fd1 ;
  reg sda_oe_fd1 ;
  reg sda_o_fd1 ;
  reg sda_i_fd1 ;
  reg scl_fd2; 
  reg scl_o_fd2 ;
  reg scl_oe_fd2 ;
  reg sda_fd2 ;
  reg sda_oe_fd2 ;
  reg sda_o_fd2 ;
  reg sda_i_fd2 ;
  reg mdio_mon ;

// WISHBONE etermac
wire    [31:0]    m_wb_adr_o;
wire     [3:0]    m_wb_sel_o;
wire              m_wb_we_o;
wire     [31:0]    m_wb_dat_i;
wire    [31:0]    m_wb_dat_o;
wire              m_wb_cyc_o;
wire              m_wb_stb_o;
wire               m_wb_ack_i;
wire               m_wb_err_i;
//
wire     [3:0]    mtxd_pad_o;
wire              mtxen_pad_o;

//MDIO
  
  wire          Mdi_I; 
  wire          Mdo_O;  
  wire          Mdo_OE; 
  wire          Mdc_O; 

  clk_gen clk_gen
   (// Clock in ports
    .CLK_IN1(clk_100m),      // IN
    // Clock out ports
    .CLK_OUT1(Clk),     // OUT
    .CLK_OUT2(clk_25m),     // OUT
    // Status and control signals
    .RESET(Reset),// IN
    .LOCKED(LOCKED));      // OUT   

 assign   reset_unlocked = ((Reset==1'b1)||(LOCKED==1'b0))?1'b1:1'b0;
   
    
  //LED
  
  //assign led = {data_out[1:0],GPO1[1:0]};
  assign led = {phy_rstb,1'b0,o_pwm1,o_pwm0};
  //
  assign lcd_co = o_pwm1;
   
  //Mڑ
  assign rstn = ~reset_unlocked;
  assign phy_rstb = ~reset_unlocked;
   
  assign GPI1 = {28'h0000000,dip_sw};
  assign data_in = data_out;
  assign data_in_en = data_out_en; 
  assign chk_data = data_out[7:0];  
  assign INTC_Interrupt = 1'b0;
   

  //ӉHIM
  assign iobus_reg_sel  = ((IO_Address>=p_reg_addr_low)&&(IO_Address<=p_reg_addr_hi))? 1'b1:1'b0;
  assign iobus_bram_sel = ((IO_Address>=p_bram_addr_low)&&(IO_Address<=p_bram_addr_hi))? 1'b1:1'b0;  
  assign iobus_wb_sel  =  ((IO_Address>=p_wb_addr_low)&&(IO_Address<=p_wb_addr_hi))? 1'b1:1'b0;
  //Ǐof[^̑I       
  assign IO_Read_Data = (iobus_reg_sel==1'b1)?  IO_Read_Data_reg:
                        (iobus_bram_sel==1'b1)? IO_Read_Data_bram:
                        (iobus_wb_sel==1'b1)?   IO_Read_Data_wb:
                                                32'h00000000;
  //ReadyM̑I  
  assign IO_Ready =    (iobus_reg_sel==1'b1)?  IO_Ready_reg:
                       (iobus_bram_sel==1'b1)? IO_Ready_bram:
                       (iobus_wb_sel==1'b1)?  IO_Ready_wb:
                                               1'b0;

//Microblaze MCS̃CX^X
mb_mcs mcs_0 (
  .Clk(Clk), // input Clk
  .Reset(reset_unlocked), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready), // input IO_Ready
  .UART_Rx(UART_Rx), // input UART_Rx
  .UART_Tx(UART_Tx), // output UART_Tx
  .FIT1_Interrupt(FIT1_Interrupt), // output FIT1_Interrupt
  .FIT1_Toggle(FIT1_Toggle), // output FIT1_Toggle
  .PIT1_Interrupt(PIT1_Interrupt), // output PIT1_Interrupt
  .PIT1_Toggle(PIT1_Toggle), // output PIT1_Toggle
  .GPO1(GPO1), // output [31 : 0] GPO1
  .GPI1(GPI1), // input [31 : 0] GPI1
  .GPI1_Interrupt(GPI1_Interrupt), // output GPI1_Interrupt
  .INTC_Interrupt(INTC_Interrupt), // input [0 : 0] INTC_Interrupt
  .INTC_IRQ(INTC_IRQ) // output INTC_IRQ
); 
 
//ݒ背WX^̃CX^X
iobus_reg
#(p_reg_addr_low, p_reg_addr_hi)
iobus_reg_0 (
  .Clk(Clk), // input Clk
  .Reset(reset_unlocked), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_reg), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_reg), // input IO_Ready
  //input 
  .data_in(data_in), // input data
  .data_in_en(data_in_en), // input data enable
  //output 
  .data_out(data_out), // output data
  .data_out_en(data_out_en) // output data enable
);

//ubNRAMC^[tF[X̃CX^X
iobus_bram
 #(p_bram_addr_low, p_bram_addr_hi)
iobus_bram_0 (
  .Clk(Clk), // input Clk
  .Reset(reset_unlocked), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_bram), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_bram), // input IO_Ready
  //BRAM Port-B                  
  .clkb(clkb), // input clkb
  .web(web), // input [3 : 0] web
  .addrb(addrb), // input [8 : 0] addrb
  .dinb(dinb), // input [31 : 0] dinb
  .doutb(doutb) // output [31 : 0] doutb
);
   
iobus2wb 
#(p_wb_addr_low, p_wb_addr_hi)
iobus2wb(
  .Clk(Clk), // input Clk
  .Reset(reset_unlocked), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_wb), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_wb), // input IO_Ready
  //wishbone MicorBlaze -> IP
  .CLK_O(WB_CLK),         // clock
  .RST_O(WB_RST),         // reset 
  .CYC_O(WB_CYC),         // cycle
  .STB_O(WB_STB),         // strobe
  .ADR_O(WB_ADR),         // address adr_i
  .SEL_O(WB_SEL),         // byte_sel
  .WE_O( WB_WE),          // write enable
  .DAT_O(WB_DAT_I),       // data output
  .DAT_I(WB_DAT_O),       // data input
  .ACK_I(WB_ACK),         // normal bus termination
  .ERR_I(WB_ERR),         // error bus termination
  .RTY_I(WB_RTY)          // retry
                  );
     
//opencores simple_gpio    
 PWM PWM_0(
  .i_wb_clk(WB_CLK),         // clock
  .i_wb_rst(WB_RST),        // reset (asynchronous active hi)
  .i_wb_cyc(cyc_i_pwm0),    // cycle
  .i_wb_stb(stb_i_pwm0),    // strobe
  .i_wb_adr(adr_i_pwm0),         // address adr_i
  .i_wb_we(we_i_pwm0),      // write enable
  .i_wb_data(WB_DAT_I[15:0]),     // data output
  .o_wb_data(dat_o_pwm0),    // data input
  .o_wb_ack(ack_o_pwm0),    // normal bus termination
  .i_extclk(i_extclk),
  .i_DC(i_DC),
  .i_valid_DC(i_valid_DC),
  .o_pwm(o_pwm0)
                         ); 
 PWM PWM_1(
  .i_wb_clk(WB_CLK),       // clock
  .i_wb_rst(WB_RST),       // reset (asynchronous active hi)
  .i_wb_cyc(cyc_i_pwm1),    // cycle
  .i_wb_stb(stb_i_pwm1),    // strobe
  .i_wb_adr(adr_i_pwm1), // address adr_i
  .i_wb_we(we_i_pwm1),      // write enable
  .i_wb_data(WB_DAT_I[15:0]),     // data output
  .o_wb_data(dat_o_pwm1),   // data input
  .o_wb_ack(ack_o_pwm1),    // normal bus termination
  .i_extclk(i_extclk),
  .i_DC(i_DC),
  .i_valid_DC(i_valid_DC),
  .o_pwm(o_pwm1)
                         );


 i2c_master_top i2c_top (
  // wishbone interface
  .wb_clk_i(WB_CLK),
  .wb_rst_i(WB_RST),
  .arst_i(rstn),
  .wb_adr_i(adr_i_i2c),
  .wb_dat_i(WB_DAT_I[7:0]),
  .wb_dat_o(dat_o_i2c),
  .wb_we_i(we_i_i2c),
  .wb_stb_i(stb_i_i2c),
  .wb_cyc_i(cyc_i_i2c),
  .wb_ack_o(ack_o_i2c),
  .wb_inta_o(inta_i2c),

// i2c signals
  .scl_pad_i(scl_pad_i),
  .scl_pad_o(scl_pad_o),
  .scl_padoen_o(scl_padoe_o),
  .sda_pad_i(sda_pad_i),
  .sda_pad_o(sda_pad_o),
  .sda_padoen_o(sda_padoe_o)
           );

  
ethmac ethmac
(
  // WISHBONE common
  .wb_clk_i(WB_CLK),
  .wb_rst_i(WB_RST),
  .wb_dat_i(WB_DAT_I),
  .wb_dat_o(dat_o_emac), 

  // WISHBONE slave
 
  .wb_adr_i(WB_ADR[11:2]),
  .wb_sel_i(WB_SEL),
  .wb_we_i(we_i_emac),
  .wb_cyc_i(cyc_i_emac), 
  .wb_stb_i(stb_i_emac),
  .wb_ack_o(ack_o_emac),
  .wb_err_o(err_o_emac),
        
// WISHBONE master
  .m_wb_adr_o(m_wb_adr_o), .m_wb_sel_o(m_wb_sel_o), .m_wb_we_o(m_wb_we_o), .m_wb_dat_i(m_wb_dat_i), 
  .m_wb_dat_o(m_wb_dat_o), .m_wb_cyc_o(m_wb_cyc_o), .m_wb_stb_o(m_wb_stb_o), .m_wb_ack_i(m_wb_ack_i), 
  .m_wb_err_i(m_wb_err_i), 

  //TX
  .mtx_clk_pad_i(MTxClk_i), .mtxd_pad_o(mtxd_pad_o), .mtxen_pad_o(mtxen_pad_o), .mtxerr_pad_o(MTxErr),

  //RX
  .mrx_clk_pad_i(MRxClk_i), .mrxd_pad_i(MRxD_i), .mrxdv_pad_i(MRxDV_i), .mrxerr_pad_i(MRxErr), 
  .mcoll_pad_i(MColl),    .mcrs_pad_i(MCrs), 
  
  // MIIM
  .mdc_pad_o(Mdc_O), .md_pad_i(Mdi_I), .md_pad_o(Mdo_O), .md_padoe_o(Mdo_OE),
  
  .int_o()
);

 assign MTxClk_i = (dip_sw[0]==1'b0)?MTxClk:clk_25m;   

 assign MTxD = mtxd_pad_o;
   
 assign MTxEn = mtxen_pad_o;
   

 assign MRxClk_i = (dip_sw[0]==1'b0)?MRxClk:clk_25m;   
 assign MRxD_i = (dip_sw[0]==1'b0)?MRxD:mtxd_pad_o;   
 assign MRxDV_i = (dip_sw[0]==1'b0)?MRxDV:mtxen_pad_o;
   
   assign Mdc = Mdc_O;
   assign Mdio = (Mdo_OE==1'b1)? Mdo_O:1'bz;
   assign Mdi_I = Mdio ;
   

   
wb_bram_if wb_bram_if
(
  // WISHBONE common
  .wb_clk_i(WB_CLK),
  .wb_rst_i(WB_RST),
  .wb_dat_i(m_wb_dat_o),
  .wb_dat_o(m_wb_dat_i), 
  // WISHBONE slave
  .wb_adr_i(m_wb_adr_o[12:2]),
  .wb_sel_i(m_wb_sel_o),
  .wb_we_i(m_wb_we_o),
  .wb_cyc_i(m_wb_cyc_o), 
  .wb_stb_i(m_wb_stb_o),
  .wb_ack_o(m_wb_ack_i),
  .wb_err_o(m_wb_err_i), 
  //BRAM Port-B                  
  .clkb(clkb), // input clkb
  .web(web), // input [3 : 0] web
  .addrb(addrb), // input [8 : 0] addrb
  .dinb(dinb), // input [31 : 0] dinb
  .doutb(doutb) // output [31 : 0] doutb
 
);


   
//WihsBoneBus interconnect
   assign wb_pwm0_sel = ((WB_ADR>=p_wb_pwm0_low)&&(WB_ADR<=p_wb_pwm0_hi))? 1'b1:1'b0;
   assign cyc_i_pwm0 =(wb_pwm0_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_pwm0 =(wb_pwm0_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_pwm0 ={12'h000,WB_ADR[3:0]};
   assign we_i_pwm0  =(wb_pwm0_sel==1'b1)?WB_WE:1'b0;
   
   assign wb_pwm1_sel = ((WB_ADR>=p_wb_pwm1_low)&&(WB_ADR<=p_wb_pwm1_hi))? 1'b1:1'b0;
   assign cyc_i_pwm1 =(wb_pwm1_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_pwm1 =(wb_pwm1_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_pwm1 ={12'h000,WB_ADR[3:0]};
   assign we_i_pwm1  =(wb_pwm1_sel==1'b1)?WB_WE:1'b0;

                         
   assign wb_i2c_sel = ((WB_ADR>=p_wb_i2c_low)&&(WB_ADR<=p_wb_i2c_hi))? 1'b1:1'b0;
   assign cyc_i_i2c =(wb_i2c_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_i2c =(wb_i2c_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_i2c =WB_ADR[2:0];
   assign we_i_i2c  =(wb_i2c_sel==1'b1)?WB_WE:1'b0;

/*                        
   assign wb_bram_sel = ((WB_ADR>=p_wb_bram_low)&&(WB_ADR<=p_wb_bram_hi))? 1'b1:1'b0;
   assign cyc_i_bram =(wb_bram_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_bram =(wb_bram_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_bram =WB_ADR[2:0];
   assign we_i_bram  =(wb_bram_sel==1'b1)?WB_WE:1'b0;
*/                        
   assign wb_emac_sel = ((WB_ADR>=p_wb_emac_low)&&(WB_ADR<=p_wb_emac_hi))? 1'b1:1'b0;
   assign cyc_i_emac =(wb_emac_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_emac =(wb_emac_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_emac =WB_ADR[2:0];
   assign we_i_emac  =(wb_emac_sel==1'b1)?WB_WE:1'b0;
   //                          
   assign WB_DAT_O =(wb_pwm0_sel==1'b1)?{dat_o_pwm0,dat_o_pwm0}:
                    (wb_pwm1_sel==1'b1)?{dat_o_pwm1,dat_o_pwm1}:
                    (wb_i2c_sel==1'b1)?{dat_o_i2c,dat_o_i2c,dat_o_i2c,dat_o_i2c}:
                    //(wb_bram_sel==1'b1)?dat_o_bram:
                    (wb_emac_sel==1'b1)?dat_o_emac:
                                        32'h00000000;
   assign WB_ACK = (wb_pwm0_sel==1'b1)?ack_o_pwm0:
                   (wb_pwm1_sel==1'b1)?ack_o_pwm1:
                   (wb_i2c_sel==1'b1)?ack_o_i2c:
                   //(wb_bram_sel==1'b1)?ack_o_bram:
                   (wb_emac_sel==1'b1)?ack_o_emac:
                                        WB_STB;
   
   assign WB_ERR = 1'b0;
   assign WB_RTY = 1'b0;
   
// PWM inputs
   assign i_extclk =1'b0;
   assign i_DC = 16'h0000;
   assign i_valid_DC =1'b0;    
   
// I2C
    assign scl = scl_padoe_o ? 1'bz : scl_pad_o;
    assign sda = sda_padoe_o ? 1'bz: sda_pad_o;
    assign scl_pad_i = scl;
    assign sda_pad_i = sda;

//mon
   
always @ (posedge Clk )
  mon <= sda_pad_i^scl_mon^scl_o_mon^scl_oe_mon^sda_mon^sda_oe_mon^sda_o_mon^mdio_mon;
   

always @ (posedge Clk  )
    begin
      scl_fd1    <= scl;
      scl_oe_fd1 <= scl_padoe_o;
      scl_o_fd1  <= scl_pad_o;
      sda_fd1    <= sda; 
      sda_oe_fd1 <= sda_padoe_o;
      sda_o_fd1  <= sda_pad_o;
    end
   
always @ (posedge Clk  )
    begin
      scl_fd2    <= scl_fd1;
      scl_oe_fd2 <= scl_oe_fd1;
      scl_o_fd2  <= scl_o_fd1;
      sda_fd2    <= sda_fd1; 
      sda_oe_fd2 <= sda_oe_fd1;
      sda_o_fd2  <= sda_o_fd1;
    end
   
always @ (posedge Clk  )
    begin
      scl_mon    <= scl_fd2;
      scl_o_mon  <= scl_o_fd2;
      scl_oe_mon <= scl_oe_fd2;
      sda_mon    <= sda_fd2;
      sda_oe_mon <= sda_oe_fd2;
      sda_o_mon  <= sda_o_fd2;
      mdio_mon  <= Mdio;
    end
//                          
endmodule
